package CTL.CCompat;

import CTL.Serialize.*;
import CTL.Types.*;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

/** Compatibility with C++ any objects.
 * 	Each class can define their AnyObj identifier by having a public
 * 	string attribute anyobj_ident.
 */
public class AnyObj implements Writable
{
	/** Contained data */
	private Object data;
	
	/** Constructor */
	public <T extends Object> AnyObj (T obj)
	{
		data = obj;
	}
	
	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return data.toString();
	}

	/** Retrieve the contained data
	 * @return Object
	 */
	public Object getData ()
	{
		return data;
	}

	/** Read from stream
	 * @param in Stream to read from
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException, 
		InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		data = in.readObject();
	}
	
	/** Write to stream
	 * @param out Stream to write to
	 */
	public void write (SerialOut out) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		out.writeObject(data);
	}
}
