package CTL.CCompat;

import CTL.Serialize.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

/** C++ array compatibility class */
public class CArray<T extends Object> implements Writable, TemplHack	
{
	/** Actual array */
	private Object[] arr;
	/** TypeTree for nested templated arrays */
	private TypeTree type = null;

	/** Template hack
	 * @param types TypeTree
	 */
	public void setTypes (TypeTree[] types)
	{
		if (types.length>0)
			type = types[0];
		else
			try
			{
				type = new TypeTree(Object.class);
			} catch (ClassNotFoundException e) {}
	}

	/** Simple constructor */
	public CArray ()
	{
	}

	/** Constructor with known array size */
	public CArray (int size)
	{
		arr = new Object[size];
	}

	/** Set a new value for one element
	 * @param idx Index
	 * @param value Value
	 */
	public void set (int idx, T value)
	{
		arr[idx] = value;
	}

	/** Get one element's value
	 * @param idx Index
	 * @return Value
	 */
	public T get (int idx)
	{
		return (T)arr[idx];
	}

	/** Conversion to string
	 * @return String
	 */
	public String toString ()
	{
		return java.util.Arrays.toString(arr);
	}

	/** Read from stream
	 * @param in Stream to read from
	 */
	public void read (SerialIn in) throws IOException, ClassNotFoundException,
		InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		arr = in.readArray(type.getType(), type.getTypeParameters());
	}
	
	/** Read from stream 
	 * @param in Stream to read from
	 * @param len Length
	 */
	public void read (SerialIn in, int len) throws IOException, 
		ClassNotFoundException, InstantiationException, 
		IllegalAccessException, InvocationTargetException
	{
		arr = in.__readArray(type.getType(), type.getTypeParameters(), len);
	}
	
	/** Write to stream
	 * @param out Stream to write to
	 */
	public void write (SerialOut out) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		out.writeArray(arr);
	}
}
