package CTL.Comm;

import CTL.*;
import CTL.Types.*;

import com.apple.dnssd.*;
import java.util.*;

/** Code for DNS service discovery support */
public class DNSSD implements BrowseListener, RegisterListener, ResolveListener
{
	private boolean isReg = false;
	private HashMap<String,Location> map = new HashMap<String,Location>();
	
	public void operationFailed (DNSSDService service, int errorCode)
	{
		Env.log.msg(Logger.ERR, "DNS-SD "+(isReg ? "Registration" : "Browse")+
			" failed (err: "+errorCode+").");
	}

	public void serviceRegistered (DNSSDRegistration reg, int flags,
		String name, String type, String domain)
	{
		Env.log.msg(Logger.INFO, "DNS-SD Registration successful (name: "+name+
			", type: "+type+", domain: "+domain);
	}

	public void serviceResolved(DNSSDService resolver, int flags, int ifIndex,
    	String fullName, String hostName, int port, TXTRecord txtRecord)
    {
    	/*for (int i = 0; i < txtRecord.size(  ); i++)
      	{
      		String key = txtRecord.getKey(i);
      		String value = txtRecord.getValueAsString(i);
      		if (key.length() > 0) 
				System.out.println("\t" + key + "=" + value);
      	}*/
	
		StringTokenizer tok = new StringTokenizer(fullName, ".");
		String name = tok.nextToken();
		map.put(name, new Location(hostName+":"+port+" dmn"));
    }

	public void serviceFound (DNSSDService browser, int flags, int ifIndex,
		String name, String type, String domain)
	{	
		try
		{
			com.apple.dnssd.DNSSD.resolve(0, 
				com.apple.dnssd.DNSSD.ALL_INTERFACES, name, type, domain, this);
		}
		catch (Exception e) {}
	}

	public void serviceLost (DNSSDService browser, int flags, int ifIndex, 
		String name, String regType, String domain)
	{
		map.remove(name);
	}

	public DNSSD () throws DNSSDException, InterruptedException
	{
		com.apple.dnssd.DNSSD.browse("_ctl._tcp", this);
	}

	public DNSSD (String name, int port) throws DNSSDException, InterruptedException
	{
		isReg = true;
		com.apple.dnssd.DNSSD.register(name, "_ctl._tcp", port, this);
	}

	public String toString ()
	{
		if (isReg)
			return "DNSSD Registration object";
		return map.toString();
	}

	public Location discover (String name)
	{
		if (isReg)
			return null;
		return map.get(name);
	}

	public static void main (String[] args)
	{
		try
		{
			Env.newLogger("/tmp/dnssd");
			DNSSD serv = new DNSSD("AddRI", 4849);
			DNSSD serv2 = new DNSSD("FooRI", 4850);
			DNSSD lookup = new DNSSD();
			Thread.sleep(5000);
			System.out.println(serv+"\n"+serv2);
			//System.out.println(lookup);
			System.out.println(lookup.discover("MooRI"));
			System.out.println(lookup.discover("FooRI"));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
