package CTL;

import java.lang.reflect.*;
import java.util.*;

/** Map of objects */
public class ObjectMap
{
	/** Number of objects created by this map */
	private int counter;
	/** Underlying LinkedList of ObjectMapEntries */
	private LinkedList<ObjectMapEntry> list;
	
	/** Generate a new map */
	public ObjectMap ()
	{
		counter=0;
		list = new LinkedList<ObjectMapEntry>();
	}

	/** Retrieve the number of generated objects
	 * @return Number of objects
	 */
	public int getCounter ()
	{
		return counter;
	}
	
	/** Create an object and insert it into the map
	 * @param c Constructor to call
	 * @param args Arguments to the constructor
	 * @return ID of the created object
	 */
	public int createObj (Constructor c, Object[] args) 
		throws InstantiationException, IllegalAccessException, 
		InvocationTargetException
	{
		Object obj = c.newInstance(args);
		int hash = obj.hashCode(); 
		ObjectMapEntry entry = new ObjectMapEntry(hash, obj);
		list.add(entry);
		counter++;
		Env.log.msg(Logger.DBG, "Created Object "+hash+".");
		Env.log.msg(Logger.DBG2, "Created "+counter+" object(s).");
		return hash;
	}
	
	/** Register a previously constructed object
	 * @param obj Object
	 * @return ID of the enqueued object
	 */
	public int regObj (Object obj)
	{
		return regObj(obj, obj.hashCode());
	}

	/** Register a previously constructed object
	 * @param obj Object
	 * @param ID ID to register the object with
	 * @return The specified ID
	 */
	public int regObj (Object obj, int ID)
	{
		ObjectMapEntry entry = new ObjectMapEntry(ID, obj);
		list.add(entry);
		counter++;
		return ID;
	}

	/** Retrieve an object by ID
	 * @param hash ID
	 * @return The object or null if none was registered for the specified
	 * ID
	 */
	public Object getObj (int hash)
	{
		ListIterator iter = list.listIterator();
		while (iter.hasNext())
		{
			ObjectMapEntry entry = (ObjectMapEntry)iter.next();
			if (entry.getHash()==hash)
				return entry.getObj();
		}
		return null;
	}

	/** Dump the map's contents to screen */
	public String dump ()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("ObjectMap dump:\n");
		for (ObjectMapEntry e : list)
			buf.append("\t\t"+e.toString()+"\n");
		buf.append("\t\tEOD");
		return buf.toString();
	}
}
