package CTL;

import CTL.Serialize.*;
import CTL.Streams.*;
import CTL.Types.*;
import NCLib.Strings;
import ReflWrap.*;

import gnu.getopt.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

/** Static utility functions which did not fit anywhere else */
public class RUtil
{
	/** Method objects for serialRead() and serialWrite() */
	public static Method serialrd, serialrd2, serialwr;
	/** Keeps track of recursive calls to avoid infinite loops */
	private static int halt;
	
	static
	{
		Class[] args = new Class[1];
		try
		{
			args[0] = Object.class;
			serialwr = SerialOut.class.getDeclaredMethod("serialWrite", args);
			args[0] = Class.class;
			serialrd = SerialIn.class.getDeclaredMethod("serialRead", args);
			args[0] = TypeTree.class;
			serialrd2 = SerialIn.class.getDeclaredMethod("serialRead", args);
		}
		catch (Exception e)
		{
			throw new RuntimeException("Critical error.");
		}
	}
	
	/** Print the backtrace of an exception
	 * @param e Exception
	 * @return String
	 */
	public static String stackTrace (Exception e)
	{
		String trace = __stackTrace(e.getStackTrace());
		String msg = "";

		if (e.getClass().getName().equals("CTL.CTLException"))
			msg = ((CTLException)e).getMsg();

		return e+": "+msg+System.getProperty("line.separator")+trace;
	}

	/** Stacktrace printer helper function
	 * @param trace Stacktrace element
	 * @return String
	 */
	private static String __stackTrace (StackTraceElement[] trace)
	{
		return __stackTrace(trace, trace.length);
	}
	
	/** Stacktrace printer helper function
	 * @param trace Stacktrace element
	 * @param len Length
	 * @return String
	 */
	private static String __stackTrace (StackTraceElement[] trace, int len)
	{
		StringBuffer buf = new StringBuffer();
		
		for (int i=0;i<len;i++)
			buf.append("\t\t"+trace[i].toString()+
				(i==trace.length-1 ? "" : System.getProperty("line.separator")));

		return buf.toString();
	}
		
	/** Pretty print stacktrace of the current thread
	 * @return String
	 */
	public static String stackTrace ()
	{
		return __stackTrace(Thread.currentThread().getStackTrace(), 5);
	}
	
	/** Properly decode and display an exception
	 * @param e Exception
	 */
	public static void except (Exception e)
	{
		Env.log.msg(Logger.DBG, e.toString());
		
		if (e.getClass().getName().equals
			("java.lang.reflect.InvocationTargetException"))
		{
			except((Exception)((InvocationTargetException)e).
				getTargetException());
			return;
		}
			
		if (e.getClass().getName().equals("java.io.StreamCorruptedException"))
		{
			Env.log.msg(Logger.ERR, stackTrace(e));
			throw new RuntimeException("Stream corrupted.");
		}
		
		if (e.getClass().getName().equals("java.io.OptionalDataException"))
		{
			Env.log.msg(Logger.ERR, "Stream is not intelligent enough.");
			Env.log.msg(Logger.ERR, stackTrace(e));
			return;
		}

		if (e.getClass().getName().equals("java.net.ConnectException"))
		{
			String msg = "Connection error: "+e.getMessage();
			System.err.println(msg);
			Env.log.msg(Logger.ERR, msg);
			Env.log.msg(Logger.ERR, stackTrace(e));
			throw new RuntimeException("Unexpected connection error.");
		}

		Env.log.msg(Logger.ERR, stackTrace(e));

		if ((e.getClass().getName().equals("java.net.SocketException"))||
			(e.getClass().getName().equals("java.io.EOFException"))||
			(e.getClass().getName().equals("java.io.NotSerializableException"))||
			(e.getClass().getName().equals("java.net.SocketTimeoutException")))
			if (!Env.isDebugger)
			{
				Env.log.msg(Logger.ERR, "Unrecoverable error.");
				System.exit(10);
			}
	}

	/** Get the FID for a certain method
	 * @param method Method
	 * @return FunctionID
	 */
	public static char getFID (Method method)
	{
		ClassInfo cinfo = new ClassInfo(method.getDeclaringClass());
		Method[] mm = cinfo.methods();

		for (int i=0;i<mm.length;i++)
			if (mm[i].equals(method))
				return (char)i;

		return (char)-1;
	}

	/** Get the FID for a certain constructor
	 * @param construct Constructor
	 * @return FunctionID
	 */
	public static char getCID (Constructor construct)
	{
		Constructor[] cc = construct.getDeclaringClass().getConstructors();

		for (int i=0;i<cc.length;i++)
			if (cc[i].equals(construct))
				return (char)i;

		return (char)-1;
	}
	
	/** Returns a reader function for the specified type
	 * @param type Type to read
	 * @return Method which can read the type
	 */
	public static Method getReader (String type)
	{
		Method m = null;
		String type2 = Refl.convert(type);
		Class[] args = null;

		if (type.charAt(type.length()-1)==']')
			type2="Array";
		if (type2.equals("Array"))
		{
			args = new Class[1];
			args[0] = Class.class;
		}

		try
		{
			m = SerialIn.class.getDeclaredMethod("read"+type2, args);
		}
		catch (NoSuchMethodException e)
		{
			try
			{
				m = DataInputStream2.class.getDeclaredMethod("read"+type2, 
					args);
			}
			catch (NoSuchMethodException e2)
			{
				return getReader("Object");
			}
		}

		return m;
	}	

	/** Returns a writer function for the specified type
	 * @param type Type to write
	 * @return Method which can write the type
	 */
	public static Method getWriter (String type)
	{
		Class[] args = new Class[1];
		Method m = null;
		String type2 = Refl.convert(type);
		args[0] = Refl.getClass(type2);

		try
		{
			m = SerialOut.class.getDeclaredMethod("write"+type2,
				args);
		}
		catch (NoSuchMethodException e)
		{
			try
			{
				m = DataOutputStream2.class.getDeclaredMethod("write"+type2,
					args);
			}
			catch (NoSuchMethodException e2)
			{
				return getWriter("Object");
			}
		}

		return m;
	}

	/** Returns a receiver function for the specified type
	 * @param type Type to receive
	 * @return Method which can receive the type
	 */
	public static Method getReceiver (String type)
	{
		Method m = null;
		Class[] args = new Class[2];
		String type2 = Refl.convert(type);

		if (!type.equals("Object"))
			halt=0;
		
		try
		{
			args[0] = Refl.getClass(type2);
			args[1] = Integer.TYPE;
			m = rResult.class.getDeclaredMethod("receive", args);
		}
		catch (NoSuchMethodException e)
		{
			if (halt>5)
				return null;
			halt++;
			return getReceiver("Object");
		}

		return m;
	}

	/** Convert a reader function to string
	 * @param reader Reader function
	 * @return String
	 */
	public static String reader2str (String reader)
	{
		StringTokenizer tok = new StringTokenizer(reader, "d");
		tok.nextToken();
		String foo = tok.nextToken(" ");
		return new String(foo.getBytes(), 1, foo.length()-1);
	}

	/** Find the remote interface class for an implementation class
	 * @param klas Implementation class
	 * @return RI class
	 */
	public static Class classToRI (Class klas) throws ClassNotFoundException
	{
		if (Strings.endsWith(klas.toString(), "Debug"))
			return klas;
		ClassInfo cinfo = new ClassInfo(klas);
		String name = "javaSys."+cinfo.name();
		if (name.endsWith("CTLI"))
			name = name.substring(0, name.length()-4);
		else
			name = name + "CI";
		return Class.forName(name);
	}

	/** Calculate the serialized size of an object
	 * @param obj Object
	 * @return Size
	 */
	public static int objSize (Object obj) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		OOStream out = new OOStream(baos);

		if (obj==null)
			return baos.toByteArray().length;
		
		ClassInfo cinfo = new ClassInfo(obj.getClass());
		Method m = RUtil.getWriter(Refl.convert(cinfo.name()));
		
		if (m.getName().equals("writeObject"))
			m = null;
		
		if ((m!=null)&&(m.getName().equals("writeArray")))
		{
			int sz=0;
			for (int i=0;i<java.lang.reflect.Array.getLength(obj);i++)
				sz+=objSize(java.lang.reflect.Array.get(obj, i));
			return sz;
		}

		if (m!=null)
		{
			Object[] args = new Object[1];
			args[0] = obj;
			m.invoke(out, args);
		}
		// TODO: Temporary hack
		/*else
		if (cinfo.implementing("CTL.Writable"))*/
			out.serialWrite(obj);
		/*else
		if (obj.getClass().isArray())
			out.writeArray(obj);
		else
		{
			Env.log.msg(Logger.ERR, "Can not determine size of Object"+
				" of type "+cinfo.fqcn()+".");
			return -1;
		}*/
		
		out.flush();
		// XXX: Reference map is static.
		//Reference.cleanup();
		int size = baos.toByteArray().length - objSize(null);
	
		Env.log.msg(Logger.DBG2, "sizeof("+obj.getClass().getName()+
			") = "+size);
		return size;
	}

	/** Retrieve a conversion code snippet for a certain type
	 * @param type Type
	 * @param convert_me Name of the variable to convert
	 * @return Code
	 */
	public static String converter (String type, String convert_me)
	{
		if (type.equals("boolean"))
			return "((Boolean)"+convert_me+").booleanValue()";
		if (type.equals("byte"))
			return "((Byte)"+convert_me+").byteValue()";
		if (type.equals("char"))
			return "((Character)"+convert_me+").charValue()";
		if (type.equals("double"))
			return "((Double)"+convert_me+").doubleValue()";
		if (type.equals("float"))
			return "((Float)"+convert_me+").floatValue()";
		if (type.equals("int"))
			return "((Integer)"+convert_me+").intValue()";
		if (type.equals("long"))
			return "((Long)"+convert_me+").longValue()";
		if (type.equals("short"))
			return "((Short)"+convert_me+").shortValue()";
		return "("+type+")"+convert_me;
	}

	/** Fault-tolerant string to integer conversion
	 * @param str String to convert
	 * @return Integer
	 */
	public static int tryInt (String str)
	{
		int i = -1;
		try
		{
			i = Integer.parseInt(str);
		} catch (NumberFormatException e) {}
		return i;
	}

	/** Extract a PeerID from a CLI option
	 * @param arg String
	 * @return PeerID
	 */
	public static PeerID optPeerID (String arg)
	{
		StringTokenizer token = new StringTokenizer(arg, ":");
		String host = "localhost";
		int port = 0;
		
		try
		{
			host = token.nextToken();
			port = tryInt(token.nextToken());
		}
		catch (NoSuchElementException e) {}
		
		if (port == -1)
			port = 0;
		
		try
		{
			return new PeerID(host, port);
		}
		catch (UnknownHostException e) {}
		
		try
		{
			return new PeerID("localhost", 0);
		}
		catch (UnknownHostException e) {}

		return null; // Never reached.
	}

	/** Extract a PeerID from a CLI option
	 * @param g Getopt parser object
	 * @return PeerID
	 */
	public static PeerID optPeerID (Getopt g) 
	{
		return optPeerID(g.getOptarg());
	}

	/** Extract an integer from a CLI option
	 * @param g Getopt parser object
	 * @return Integer
	 */
	public static int optInt (Getopt g)
	{
		return tryInt(g.getOptarg());
	}

	/** Extract a String from a CLI option
	 * @param g Getopt parser object
	 * @return String
	 */
	public static String optString (Getopt g)
	{
		return g.getOptarg();
	}

	// TODO: document
	public static String getBase (String name)
	{
		if (Strings.endsWith(name, "RI"))
			name = name.substring(0, name.length()-2)+"CI";

		try
		{
			Constructor c = Class.forName(name).getConstructor(Object.class);
			RI ri = (RI)c.newInstance(new Object[1]);
			return ri.getBase();
		}
		catch (Exception e)
		{
			//e.printStackTrace();
		}
		
		return null;
	}
	
	public static void main (String[] args)
	{
		System.out.println(optPeerID("localhost:347784"));
		System.out.println(optPeerID("localhost:foo"));
		System.out.println(optPeerID("foo"));
		System.out.println(tryInt("737834"));
		System.out.println(tryInt("foo"));
	}
}
