package CTL.Streams;

import CTL.*;
import CTL.Serialize.*;
import CTL.Types.*;

import java.io.*;
import java.lang.reflect.*;
import java.nio.*;

/** ObjectOutputStream clone for the CTL */
public class OOStream extends SerialOut implements OutStream
{
	public enum Serial { CTL, ASCII };
	private boolean swapBytes = false;
	private SerialOut stream;
	
	/** Generate a new stream
	 * @param out Underlying output stream
	 */
	public OOStream (OutputStream out) throws IOException, CTLException
	{
		this(out, Serial.CTL);
	}

	/** Constructor */
	public OOStream (OutputStream out, Serial type) throws CTLException,
		IOException
	{
		super(out);
		
		switch (type)
		{
			case CTL:
				stream = new CTLOut(out);
				break;
			case ASCII:
				File f = new File("/tmp/ctl_out.log");
				FileOutputStream out2 = new FileOutputStream(f);
				stream = new ASCIIOut(out, out2);
				break;
			default:
				throw new CTLException("Serializer does not exist.");
		}
	}

	public void changeEndianess (ByteOrder order)
	{
		if (order == ByteOrder.LITTLE_ENDIAN)
			swapBytes = true;
		else
			swapBytes = false;
	}

	public void serialWrite (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		stream.serialWrite(data);
	}

	public void flush () throws IOException
	{
		stream.flush();
	}

	public void close () throws IOException
	{
		stream.close();
	}

	public void writeString (String s) throws IOException
	{
		stream.writeString(s);
	}

	public <T extends Object> void writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException
	{
		stream.writeArray(data);
	}

	public void writeBytes (String s) throws IOException
	{
		stream.writeBytes(s);
	}

	public void writeLong (long v) throws IOException
	{
		if (swapBytes)
			v = Long.reverseBytes(v);
		stream.writeLong(v);
	}

	public void writeInt (int v) throws IOException
	{
		if (swapBytes)
			v = Integer.reverseBytes(v);
		stream.writeInt(v);
	}

	public void write (byte[] b, int off, int len) throws IOException
	{
		stream.write(b, off, len);
	}
	
	public void writeObject (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		stream.writeObject(data);
	}
	
	public <T extends Object> void __writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException
	{
		stream.__writeArray(data);
	}

	public void writeWString (String s) throws IOException
	{
		stream.writeWString(s);
	}

	public void writeUTF (String s) throws IOException
	{
		stream.writeUTF(s);
	}

	public void writeChars (String s) throws IOException
	{
		stream.writeChars(s);
	}

	public void writeDouble (double v) throws IOException
	{
		if (swapBytes)
			v = LittleEndian.doubleReverseBytes(v);
		stream.writeDouble(v);
	}

	public void writeFloat (float v) throws IOException
	{
		if (swapBytes)
			v = LittleEndian.floatReverseBytes(v);
		stream.writeFloat(v);
	}

	public void writeChar (int v) throws IOException
	{
		if (swapBytes)
			v = Integer.reverseBytes(v);
		stream.writeChar(v);
	}

	public void writeShort (int v) throws IOException
	{
		if (swapBytes)
			v = Integer.reverseBytes(v);
		stream.writeShort(v);
	}

	public void writeByte (int v) throws IOException
	{
		if (swapBytes)
			v = Integer.reverseBytes(v);
		stream.writeByte(v);
	}

	public void writeBoolean (boolean v) throws IOException
	{
		stream.writeBoolean(v);
	}

	public void write (byte[] b) throws IOException
	{
		stream.write(b);
	}

	public void write (int i) throws IOException
	{
		if (swapBytes)
			i = Integer.reverseBytes(i);
		stream.write(i);
	}

	protected void writeData (Object data) throws IOException,
		IllegalAccessException, InvocationTargetException, CTLException
	{
		Env.log.msg(Logger.ERR, "Dummy method called.");
		throw new CTLException("Dummy method.");
	}
}
