package CTL;

import ReflWrap.*;
import NCLib.*;

import java.io.*;
import java.net.*;
import java.util.*;

/** Base class for CTL tools */
public abstract class ToolBase
{
	protected enum Type { UNDEF, FILE, JAR };
	
	/** Determines where a loaded class actually lives in the filesystem
	 * @return Resource location
	 */
	protected String whereAmI ()
    {
        String clname = ByteCode.sed(this.getClass().getName(), "\\.", "/")+".class";
        URL url = ClassLoader.getSystemResource(clname);
        return url.toString();
    }

	/** Determines where a loaded class  actually lives in the filesystem
	 * @param type Filter for a specific location type
	 * @return Resource location
	 */
	protected String whereAmI (Type type)
	{
		String res = whereAmI();
		
		switch (type)
		{
			case FILE:
				if (res.startsWith("file:"))
				{
					StringTokenizer tok = new StringTokenizer(res, ":");
					tok.nextToken();
					return ByteCode.sed(tok.nextToken(), "/build.*", "");
				}
		}

		return null;
	}

	// TODO: document
	protected static LinkedList<String> findCIs ()
	{
		return findCIs("javaSys");
	}
	
	// TODO: document
	protected static LinkedList<String> findCIs (String dirname)
	{
		LinkedList<String> ret = new LinkedList<String>();
		for (File f : new File(dirname).listFiles(new CodeGen.GeneratedFluffFilter()))
		{
			String fname = ByteCode.sed(f.toString(), "\\.java", "");
			fname = ByteCode.sed(fname, "/", "\\.");
			ret.add(fname);
		}
		return ret;
	}

	// TODO: document
	protected static LinkedList<String> findImpls ()
	{
		return findClasses("Impl");
	}

	// TODO: document 
	protected static LinkedList<String> findClasses (String dirname)
	{
		LinkedList<String> ret = new LinkedList<String>();
		if (!new File(dirname).exists())
			return ret;
		for (File f : new File(dirname).listFiles())
		{
			String clname = f.toString();
			if (Strings.endsWith(clname, ".svn"))
				continue;
			if (Strings.endsWith(clname, ".cvsignore"))
				continue;
			if (Strings.endsWith(clname, "CVS"))
				continue;
			if (f.isDirectory())
			{
				ret.addAll(findClasses(clname));
				continue;
			}
			clname = ByteCode.sed(clname, "\\.class", "");
			clname = ByteCode.sed(clname, "\\.java", "");
			clname = ByteCode.sed(clname, "/", "\\.");
			clname = ByteCode.sed(clname, "\\\\", "\\.");
			ret.add(clname);
		}
		return ret;
	}

	protected static void loadClasses (LinkedList<String> files)
	{
		if (files == null)
			return;

		for (String clname : files)
		{
			try
			{
				Class moongoo = Class.forName(clname);
			}
			catch (ClassNotFoundException e)
			{
				e.printStackTrace();
			}
		}
	}

	public static LinkedList<String> findAllCIs ()
	{
		LinkedList<String> res = new LinkedList<String>();
		for (Class c : Classes.getClasses())
			if (c.getSuperclass() == RI.class)
				if (res.indexOf(c.getName()) == -1)
					res.add(c.getName());
		return res;
	}
}
