package CodeGen;

import CTL.Annotate.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;

/** Code generator for the component interface */
public class GenCI extends Gen
{
	private String clname, libname;
	
	public GenCI (String klass) throws ClassNotFoundException
	{
		super(klass);
		name = classNamer(cinfo, "CI");
		supername = "RI";
		clname = classNamer(cinfo, "RI");
		libname = classNamer(cinfo, "Local");
		//modifiers = "abstract";
	}
	
	protected void gen (OutputStreamWriter ostr) throws IOException, NoSuchMethodException
	{
		writeTo(ostr, genHead());
		writeTo(ostr, "private static CTL.Process proc2 = null;\n");
		writeTo(ostr, "private "+name+" self = null;\n");
		writeTo(ostr, genConstructors());
		writeTo(ostr, genWrappers());
		writeTo(ostr, genAccept());
		writeTo(ostr, genGetProc());
		writeTo(ostr, "public PeerID peerID ()\n"+
			"{\n"+
			"if (self != null)\n"+
			"return self.peerID();\n"+
			"return home;\n"+
			"}\n");
		writeTo(ostr, "public static void use (CTL.Process prc)\n"+
			"{\n"+
			"if (!prc.isAlive())\n"+
			"new RuntimeException(\"Process \"+prc+\" not running.\");\n"+
			"proc2 = prc;\n"+
			"}\n");
		if (!cinfo.hasMethod("toString"))
			writeTo(ostr, "public String toString ()\n"+
					"{\n"+
					"if (proc2 != null)\n"+
					"return proc2.toString();\n"+
					"if (proc != null)\n"+
					"return proc.toString();\n"+
					"return null;\n"+
					"}\n");
		writeTo(ostr, "}\n");
	}
	
	protected String genGetProc () throws IOException
	{
		StringBuffer ret = new StringBuffer();
		
		ret.append("protected static CTL.Process proc ()\n{\n");
		ret.append("if (proc2 != null)\n{\n");
		ret.append("return proc2;\n}\n");
		ret.append("if (proc == null)\n{\n");
		ret.append("String CTL_LOC = System.getenv(\"CTL_LOC\");\n");
		ret.append("if (CTL_LOC != null)\n{\n");
		ret.append("Location loc = null;\n");
		ret.append("try\n{\n");
		ret.append("loc = new Location(CTL_LOC);\n");
		ret.append("}\ncatch (RuntimeException e)\n{\n");
		ret.append("System.out.println(\"CTL_LOC = '\"+CTL_LOC+\"' is not a valid location.\");\n");
		ret.append("System.exit(2);\n}\n");
		ret.append("CTL.Process proc = new CTL.Process(loc);\n");
		ret.append("CTL_Locator.use(proc);\n");
		ret.append("CTL_Locator obj = new CTL_Locator();\n");
		ret.append("ClassInfo cinfo = new ClassInfo("+name+".class);\n");
		ret.append("loc = obj.get(cinfo.fqcncs(), new AnyObj(0));\n");
		ret.append("if (loc != null)\n");
		ret.append("proc = new CTL.Process(loc);\n");
		ret.append("}\nelse\n{\n");
		ret.append("// El-cheapo resource-management\n");
		ret.append("LinkedList<Location> locs = Location.parseFile(\"locs.txt\");\n");
		ret.append("proc = new CTL.Process(locs.get(0)); // FIXME: evil hardcoded value\n");
		ret.append("}\n");
		ret.append("if (proc == null)\n{\n");
		ret.append("System.out.println(\"No matching resource found.\");\n");
		ret.append("System.exit(4);\n}\n");
		ret.append("}\nreturn proc;\n}\n");

		return ret.toString();
	}

	protected String genConstructors () throws IOException
	{
		StringBuffer ret = new StringBuffer();

		for (Konstruktor con : constr)
		{
			ret.append("public "+name+" ("+con.params()+"\n");
            ret.append("{\nthis(proc()"+((con.len()==0) ? "" : ", "));
            String arglist = "";
            for (int j=0;j<con.len();j++)
                arglist += "arg"+j+((j==con.len()-1) ? "" : ", ");
            ret.append(arglist);
            ret.append(");\n}\n");

            ret.append("public "+name+" (CTL.Process proc"+
                ((con.len()==0) ? "" : ", ")+con.params()+"\n");
            ret.append("{\nswitch (proc().loc().linkage())\n{\n");
			ret.append("case Location.DMN:\n"+newObj(clname, arglist));
            ret.append("case Location.TCP:\n"+newObj(clname, arglist));
            ret.append("case Location.LIB:\n"+newObj(libname, arglist, false));
            ret.append("}\n");
			// TODO: Throw an exception!
            ret.append(linkErr());
            ret.append("}\n");
		}
		
		ret.append("public "+name+" (Object obj)\n{\n}\n");

		return ret.toString();	
	}
	
	protected String genWrappers () throws IOException, NoSuchMethodException
	{
		StringBuffer ret = new StringBuffer();

		for (int i=0;i<methods.length;i++)
		{
			Methode meth = methods[i];
			
			if ((meth == null) || (!meth.here()) || (meth.dont()))
				continue;
			
			for (int rres=0;rres<2;rres++)
			{
				boolean rr = rres==1;

				if (meth.isVoid() && rr)
					continue;
				
				String head = meth.head(rr);
				
				if (meth.isVoid())
				{
					if (meth.isStatic())
					{
						String name = meth.name();
						if (rr)
							name = name + "_rr";
						ret.append(head+"\n{\n"+
							"switch (proc().loc().linkage())\n{\n"+
							"case Location.DMN:\n"+
							newMethod(clname, name, meth.len(), false)+"return;\n"+
							"case Location.TCP:\n"+
							newMethod(clname, name, meth.len(), false)+"return;\n"+
							"case Location.LIB:\n"+
							newMethod(libname, name, meth.len(), false)+"return;\n"+
							"}\n");
						ret.append(linkErr());
						ret.append("}\n");
					}
					else
					{
						ret.append(head+"\n{\n");
						ret.append(newMethod("self", meth.name()+(rr ? "_rr" : ""), 
							meth.len(), false));
						ret.append("}\n");
					}
				}
				else
				{
					if (meth.isStatic())
					{
						String name = meth.name();
						if (rr)
							name = name + "_rr";
						ret.append(head+"\n{\n"+
							"switch (proc().loc().linkage())\n{\n"+
							"case Location.DMN:\n"+newMethod(clname, name, meth.len())+
							"case Location.TCP:\n"+newMethod(clname, name, meth.len())+
							"case Location.LIB:\n"+newMethod(libname, name, meth.len())+
							"}\n");
						ret.append(linkErr());
						ret.append("return "+(rr ? "null" : 
							Refl.defaultVal(meth.returns()))+";\n}\n");
					}
					else
					{
						ret.append(head+"\n{\n");
						ret.append(newMethod("self", meth.name()+(rr ? "_rr" : ""), 
							meth.len()));
						ret.append("}\n");
					}
				}
			}
		}

		return ret.toString();
	}
	
	/** Generate the accept() function
	 * @return Code
	 */
	protected String genAccept () throws IOException
	{
		return genAccept(this);
	}
	
	protected static String genAccept (Gen self) throws IOException
	{
		StringBuffer ret = new StringBuffer();
		StringBuffer ret2 = new StringBuffer();
		int j, st=1, me=1;
		boolean _const = self.cinfo.isAnot(const_.class);

		ret.append("@SuppressWarnings(\"unchecked\")\n");
		ret.append("public static void accept (OIStream in, Header ohead, long objID, FID fid, int id, Env env) throws IOException, ClassNotFoundException, InstantiationException, IllegalAccessException, NoSuchMethodException, InvocationTargetException, CTLException\n");
		ret.append("{\nClass[] args2 = null;\n");
		ret.append("Header head = null;\n");
		ret.append("long ID = 0;\n");
		ret.append("int tag = 0;\n");
		ret.append("IStream2 args = null;\n");
		ret.append("rPointer rp = null;\n");
		ret.append("OOStream out = null;\n");
		ret.append("IPaddr host = env.comm[0].pid().host();\n");
		ret.append("int port = env.comm[0].pid().port();\n");
		ret.append("env.log.msg(Logger.DBG2, ohead+\" \"+objID);\n");
		ret.append("if (objID==0)\n{\n");
		ret.append("char c = fid.name().charAt(fid.name().length()-1);\n");
		ret.append("env.log.msg(Logger.DBG2, new Integer(c).toString());\n");
		ret.append("env.log.msg(Logger.DBG2, fid.toString());\n");	
		ret.append("switch (c)\n{\n");
		ret.append("case 'C':\n");
		ret.append("switch (fid.ID())\n{\n");

		// Constructors
		for (int i=0;i<self.cinfo.constructors().length;i++)
		{
			Constructor cnst = self.cinfo.constructors()[i];
			if (cnst.isAnnotationPresent(cdont.class))
				continue;
			ConstructInfo cinfo2 = new ConstructInfo(cnst);
			
			ret.append("case "+i+":\n");
			if (cinfo2.params().length<1)
				ret.append("Object[] foo"+i+" = null;\n");
			else
			{
				ret.append("Object[] foo"+i+" = new Object["+
					cinfo2.params().length+"];\n");
				for (j=0;j<cinfo2.params().length;j++)
					genArgList(ret, cinfo2.params()[j], i, j, true, 
						cinfo2.getParamAnot(j));	

				// TODO: Check genArgList() for constructors
				/*{
					String p = Refl.convert(cinfo2.params()[j]);
					String reader = RUtil.getReader(p).getName();
					if (p.equals("Int"))
						p = "Integer";
					ret.append("foo"+i+"["+j+"] = (Object)");
					ret.append("new "+p+"(in."+reader+"());\n");
				}*/
			}
			ret.append("rp = in.serialRead(rPointer.class);\n");
			ret.append("env.log.msg(Logger.DBG, \"\"+rp);\n");
			ret.append("tag = in.readInt();\n");
			if (cinfo2.params().length<1)
			{
				ret.append("args2 = null;\n");
			}
			else
			{
				ret.append("args2 = new Class["+
					cinfo2.params().length+"];\n");
				for (j=0;j<cinfo2.params().length;j++)
				{
					String s = Refl.convert(cinfo2.params()[j]);
					ret.append("args2["+j+"] = ");
					if (Refl.getClass(s)==null)
						ret.append(s+".class");
					else
						ret.append("Refl.getClass(\""+s+"\")");
					ret.append(";\n");
				}
			}
			
			ret.append("Constructor c"+i+" = "+self.getFQCN()+".class.\n");
			ret.append("getDeclaredConstructor(args2);\n");
			ret.append("ID = env.map.createObj(c"+i+", foo"+i+");\n");
			ret.append("args = new IStream2();\n");
			ret.append("args.write(new rPointer(ID, env.grp.myInfo()));\n");
			ret.append("head = new Header(0, tag, env.grp.myInfo());\n");
			ret.append("Remote.answer(env.comm[id], head, "+
				"rp.objID(), new Except(),\n");
			ret.append("args);\n");
			ret.append("break;\n");
		}
		
		ret.append("}\nbreak;\ncase 'S':\n");
		ret.append("switch (fid.ID())\n{\n");
		
		// Methods
		for (int i=0;i<self.cinfo.methods().length;i++)
		{
			Method moo = self.cinfo.methods()[i];
			// If there are unassigned FIDs, skip them.
			if (moo == null)
				continue;
			MethodInfo minfo = new MethodInfo(moo);
			
			if (moo.isAnnotationPresent(dont.class))
			{
				if (minfo.isStatic())
					st++;
				else
					me++;
				
				continue;
			}
			
			if (minfo._class()!=self.cinfo._class())
				continue;
				
			StringBuffer loc;
			if (minfo.isStatic())
			{
				loc = ret;
				loc.append("case "+st+":\n");
				st++;
			}
			else
			{
				loc = ret2;
				loc.append("case "+me+":\n");
				me++;
			}
			
			for (j=0;j<minfo.params().length;j++)
				genArgList(loc, minfo.params()[j], i, j, false, 
					minfo.getParamAnot(j));

			if (!moo.isAnnotationPresent(builtin.class))
			{
				loc.append("rp = in.serialRead(rPointer.class);\n");
				//loc.append("env.log.msg(Logger.DBG, \"\"+rp);\n");
			}
			loc.append("tag = in.readInt();\n");
			
			if (minfo.isStatic()==true)
			{
				loc.append(((minfo.returnsT().equals("void")) ? ""
					: minfo.returnsT()+" res"+i+" = ")+self.getFQCN()+"."+
					minfo.name()+"(");
			}
			else
			{
				loc.append(self.getFQCN()+" foo"+i+" = ("+self.getFQCN()+
					")env.map.getObj((int)objID);\n");
				loc.append(((minfo.returnsT().equals("void")) ? ""
					: minfo.returnsT()+" res"+i+" = ")+"foo"+
					i+"."+minfo.name()+"(");
			}
				
			for (j=0;j<minfo.params().length;j++)
				loc.append("arg"+j+i+((j==minfo.params().length-1) ? ");\n" 
					: ", "));
			if (j==0)
				loc.append(");\n");
				
			loc.append("env.log.msg(Logger.DBG, \"Method "+minfo.name());
			loc.append(" was called with arguments\"+");
			for (j=0;j<minfo.params().length;j++)
				loc.append("\" arg"+j+" = "+"\"+arg"+j+i+"+\",\"+");
			if (!minfo.returns().name().equals("void"))
				loc.append("\" and result \"+res"+i+"+\".\");\n");
			else
				loc.append("\".\");\n");
			
			if (!minfo.returns().name().equals("void"))
			{
				loc.append("args = new IStream2();\n");

				if (!_const)
				{
					for (int k=0;k<minfo.getParams().length;k++)
						if (!Refl.hasAnot("@CTL.Annotate.const_()", 
								minfo.getParamAnot(k)))
							loc.append("args.write(arg"+k+i+");\n");
				}

				loc.append("args.write(res"+i+");\n");
				loc.append("head = new Header(0, tag, env.grp.myInfo());\n");
				loc.append("Remote.answer(env.comm[id], head, rp.objID(),"+
					" new Except(), args);\n");
			}
			
			loc.append("break;\n");
		}
			
		ret.append("}\nbreak;\n}\n");
		ret.append("}\nelse\n{\n");
		ret.append("env.log.msg(Logger.DBG2, fid.toString());\n");
		ret.append("switch (fid.ID())\n{\n");
		ret.append(__putRefCount());
		ret.append(ret2.toString());
		ret.append("}\n}\n");
		ret.append("env.log.msg(Logger.DBG2, \"Message Tag: \"+tag);\n");
		ret.append("}\n");

		return ret.toString();
	}
}
