package DotClass;

import ReflWrap.*;
import java.util.*;

/** Entry in a ClassList, storing information about a Class */
public class ClassListEntry
{
	/** Underlying ClassInfo object */
	private ClassInfo cinfo;
	/** Name of the class */
	private String name;
	/** Normalized name as returned by genDotName() */
	private String dot_name;
	/** Name of the superclass */
	private String sclass;
	/** Names of the interfaces */
	private String[] ifaces;

	/** Generate a new ClassListEntry
	 * @param cinfo Class information
	 */
	public ClassListEntry (ClassInfo cinfo)
	{
		this.cinfo = cinfo;
		name = cinfo.fqcn();
		dot_name = genDotName();
		sclass = null;
		ifaces = null;
	}

	/** Retrieve the name of this class
	 * @return String
	 */
	public String name ()
	{
		return name;
	}

	/** Retrieve the normalized name of this class
	 * @return String
	 */
	public String dot_name ()
	{
		return dot_name;
	}

	/** Generate a normalized name
	 * @return String
	 */
	private String genDotName ()
	{
		StringTokenizer str = new StringTokenizer(name, ".");
		String Name = null;
		while (str.hasMoreTokens())
			Name = str.nextToken();
		return Name.toLowerCase();
	}

	/** Retrieve a string representation of this object
	 * @return String
	 */
	public String toString ()
	{
		return name+", Superclass: "+sclass+", Interfaces: "+printIFaces();
	}

	/** Pretty printer for the interfaces this class implements
	 * @return String
	 */
	public String printIFaces ()
	{
		if (ifaces==null)
			return "";
		StringBuffer buf = new StringBuffer();
		for (int i=0;i<ifaces.length;i++)
			buf.append(printIFace(i)+" ");
		return buf.toString();
	}

	/** Check whether or not this class is an interface
	 * @return True if this class is an interface, false otherwise
	 */
	public boolean isIFace ()
	{
		if (sclass==null)
			return true;
		return false;
	}

	/** Print one interface
	 * @param i Index number
	 * @return String
	 */
	public String printIFace (int i)
	{
		return ifaces[i];
	}

	/** Retrieve the number of interfaces
	 * @return Number of interfaces this class implements
	 */
	public int numIFaces ()
	{
		if (ifaces==null)
			return 0;
		return ifaces.length;
	}

	/** Retrieve the superclass 
	 * @return String
	 */
	public String sclass ()
	{
		return sclass;
	}

	/** Resolve this symbol
	 * @param list List of classes
	 */
	public void resolve (ClassList list)
	{
		if (cinfo.sclass()!=null)
			sclass = list.find(cinfo.fqsuperclass(), false);

		ifaces = new String[cinfo.interfaces().length];
		for (int i=0;i<cinfo.interfaces().length;i++)
			ifaces[i] = list.find(cinfo.interfaces()[i].getName(), false);
	}		
}
