package ReflWrap;

import java.lang.annotation.*;
import java.lang.reflect.*;
import java.util.*;

/** This class wraps all information about a constructor */
public class ConstructInfo
{
	/** Underlying Constructor object */
	private Constructor c;
	/** Name */
	private String name;
	/** String representation of the parameter types */
	private String params[];

	/** Retrieve the underlying Constructor object
	 * @return Constructor object
	 */
	public Constructor c ()
	{
		return c;
	}

	/** Retrieve the constructor's name
	 * @return Name
	 */
	public String name ()
	{
		return name;
	}

	/** Retrieve the parameters this constructor takes
	 * @return Array of parameter types
	 */
	public String[] params ()
	{
		return params;
	}
	
	/** Generate from an existing constructor
	 * @param construct Constructor
	 */
	public ConstructInfo (Constructor construct)
	{
		c=construct;
		name=c.getName();
		
		Class p[] = c.getParameterTypes();
		params = new String[p.length];
		for (int i=0;i<p.length;i++)
		{
			StringTokenizer str = new StringTokenizer(p[i].getName(), ".");
			params[i] = null;
			while (str.hasMoreTokens())
				params[i] = str.nextToken();

			int len = params[i].length();
			if (params[i].substring(len-1, len).equals(";"))
				params[i] = params[i].substring(0, len-1)+"[]";
		}
	}
	
	/** Return a String representation of this object
	 * @return String
	 */
	public String toString ()
	{
		StringBuffer ret = new StringBuffer();
		
		ret.append(modifiersToString());
		ret.append(name+ " (");
		ret.append(paramsToString());
		
		return ret.toString();
	}

	/** Convert the parameters to a pretty String
	 * @return String
	 */
	public String paramsToString ()
	{
		StringBuffer ret = new StringBuffer();
		int i;
		
		for (i=0;i<params.length;i++)
			ret.append(params[i]+" arg"+i+ 
					((i==params.length-1) ? ")" : ", "));
			
		if (i==0)
			ret.append(")");

		return ret.toString();
	}	
	
	/** Convert all modifiers to a pretty String
	 * @return String
	 */
	public String modifiersToString ()
	{
		StringBuffer sb = new StringBuffer();
		int mod = c.getModifiers();

		if ((mod & Modifier.PUBLIC) != 0)
			sb.append("public ");
		if ((mod & Modifier.PRIVATE) != 0)
			sb.append("private ");
		if ((mod & Modifier.PROTECTED) != 0)
			sb.append("protected ");
		if ((mod & Modifier.ABSTRACT) != 0)
			sb.append("abstract ");
		if ((mod & Modifier.STATIC) != 0)
			sb.append("static ");
		if ((mod & Modifier.FINAL) != 0)
			sb.append("final ");
		if ((mod & Modifier.TRANSIENT) != 0)
			sb.append("transient ");
		if ((mod & Modifier.VOLATILE) != 0)
			sb.append("volatile ");
		if ((mod & Modifier.NATIVE) != 0)
			sb.append("native ");
		if ((mod & Modifier.SYNCHRONIZED) != 0)
			sb.append("synchronized ");
		if ((mod & Modifier.INTERFACE) != 0)
			sb.append("interface ");

		return sb.toString();
	}
	
	/** Get the Annotations for a certain parameter
	 * @param i Index of the requested parameter
	 * @return Array of Annotations
	 */
	public Annotation[] getParamAnot (int i)
	{
		Annotation[][] aa = c.getParameterAnnotations();
		return aa[i];
	}
}
